proc FselResources {} {
    # The "startup" priority lets users override these in ~/.Xdefaults
    # path is used to enter the file name
    option add *Fsel*font                 10x20   startup
    option add *Fsel*path.relief		sunken	startup
    option add *Fsel*path.background	white	startup
    option add *Fsel*path.foreground	black	startup
    # Text for the label on pathname entry
    option add *Fsel*l.text		File:	startup
    # Text for the OK and Cancel buttons
    option add *Fsel*ok*text		OK	startup
    option add *Fsel*ok*underline		0	startup
    option add *Fsel*cancel.text		Cancel	startup
    option add *Fsel*cancel.underline 	0	startup
    # Size of the listbox
    option add *Fsel*list.width		30	startup
    option add *Fsel*list.height		15	startup
}

# Fsel_Select returns the selected pathname, or {}
# mustExist forces the dialog to find an existing file.
proc Fsel_Select {{why "File Selection"} {default {}} {mustExist 1} } {
	global fsel

	set t [toplevel .fsel -bd 4 -class Fsel]
	FselResources

        #wm geometry $t +440+150
	wm transient $t .
	set xpos [expr [winfo rootx .]+[winfo width  .]/4]
	set ypos [expr [winfo rooty .]+[winfo height .]/4]
	wm geometry $t +${xpos}+${ypos}

	message $t.msg -aspect 1000 -text $why
	pack $t.msg -side top -fill x
        
        grab $t

	# Create a read-only entry for the current directory
	set fsel(dirEnt) [entry $t.dir -width 15 \
		-relief flat -state disabled]
	pack $t.dir -side top -fill x
    
	# Create an entry for the pathname
	# The value is kept in fsel(path)
	frame $t.top
	set e [entry $t.top.path \
		-textvariable fsel(path)]
	pack $t.top -side top -fill x

        # The label on the entry is defined with an X resource
	label $t.top.l -padx 0
	pack $t.top.l -side left
	pack $t.top.path -side right -fill x -expand true
    
	# Create a listbox to hold the directory contents
	set lb [listbox $t.list \
		-yscrollcommand [list $t.scroll set]]
	scrollbar $t.scroll -command [list $lb yview]

	# Create the OK and Cancel buttons
        # The button text is defined with an X resource
	# The OK button has a rim to indicate it is the default
	frame $t.buttons -bd 10
	frame $t.buttons.ok -bd 2 -relief sunken
	set ok [button $t.buttons.ok.b \
		-command FselOK]
	set can [button $t.buttons.cancel \
		-command FselCancel]

	# Pack the list, scrollbar, and button box
	# in a horizontal stack below the upper widgets
	pack $t.list -side left -fill both -expand true
	pack $t.scroll -side left -fill y
	pack $t.buttons -side left -fill both
	pack $t.buttons.ok $t.buttons.cancel \
		-side top -padx 10 -pady 5
	pack $t.buttons.ok.b -padx 4 -pady 4

	FselBindings $t $e $lb $ok $can

	# Initialize variables and list the directory
	if {[string length $default] == 0} {
		set fsel(path) {}
		set dir [pwd]
	} else {
		set fsel(path) [file tail $default]
		set dir [file dirname $default]
	}
	set fsel(dir) {}
	set fsel(done) 0
	set fsel(mustExist) $mustExist

	# Wait for the listbox to be visible so
	# we can provide feedback during the listing 
	tkwait visibility .fsel.list
	FselList $dir

	tkwait variable fsel(done)
	destroy $t
	return $fsel(path)
}

proc FselBindings { t e lb ok can } {
	# t - toplevel
	# e - name entry
	# lb - listbox
	# ok - OK button
	# can - Cancel button

	# Elimate the all binding tag because we
	# do our own focus management
	foreach w [list $e $lb $ok $can] {
	    bindtags $w [list $t [winfo class $w] $w]
	}
	# Dialog-global cancel binding
	bind $t <Control-c> FselCancel

	# Entry bindings
	bind $e <Return> FselOK
	bind $e <space> FselComplete

	# A single click, or <space>, puts the name in the entry
	# A double-click, or <Return>, selects the name
	bind $lb <space> "FselTake %W ; focus $e"
	bind $lb <Button-1> \
		"FselClick %W %y ; focus $e"
	bind $lb <Return> "FselTake %W ; FselOK"
	bind $lb <Double-Button-1> \
		"FselClick %W %y ; FselOK"

	# Focus management.  	# <Return> or <space> selects the name.
	bind $e <Tab> "focus $lb ; $lb select set 0"
	bind $lb <Tab> "focus $e"

	# Button focus.  Extract the underlined letter
	# from the button label to use as the focus key.
	foreach but [list $ok $can] {
		set char [string tolower [string index  \
			[$but cget -text] [$but cget -underline]]]
		bind $t <Alt-$char> "focus $but ; break"
	}
	bind $ok <Tab> "focus $can"
	bind $can <Tab> "focus $ok"

	# Set up for type in
	focus $e
}

proc FselList { dir {files {}} } {
	global fsel

	# Update the directory display
	set e $fsel(dirEnt)
	$e config -state normal
	$e delete 0 end
	$e insert 0 $dir
	$e config -state disabled
	# scroll to view the tail end
	$e xview moveto 1

	set fsel(dir) $dir
	.fsel.list delete 0 end
        # Check for bogus names
	if ![file isdirectory $dir] {
	    .fsel.list insert 0 "Bad Directory"
	    return
	}
        # Give the user some feedback
	.fsel.list insert 0 Listing...
	update idletasks

	.fsel.list delete 0
	if {[string length $files] == 0} {
		# List the directory and add an
		# entry for the parent directory
		set files [glob -nocomplain $fsel(dir)/*]
		.fsel.list insert end ../
	}

	# Sort the directories to the front
	set dirs {}
	set others {}
	foreach f [lsort $files] {
		if [file isdirectory $f] {
			lappend dirs [file tail $f]/
		} else {
			lappend others [file tail $f]
		}
	}
	foreach f [concat $dirs $others] {
		.fsel.list insert end $f
	}
}

proc FselOK {} {
	global fsel

	# Trim ../ (the parent) out of the pathname
	if {[regsub {^\.\./?} $fsel(path) {} newpath] != 0} {
		set fsel(path) $newpath
		set fsel(dir) [file dirname $fsel(dir)]
		FselOK
		return
	}
        # Assume path is relative to current directory
        # If $fsel(dir) = "/", don't need to add another slash
        if {[string compare $fsel(dir) "/"] == 0} {
                set path [string trimright \
                        $fsel(dir)$fsel(path) /]
        } else {
                set path [string trimright \
                        $fsel(dir)/$fsel(path) /]
        }
        # When the root dir is reached, path="" and needs to be set to "/"
	if {[string length $path] == 0} {
                set path "/"
        }
	if [file isdirectory $path] {
		set fsel(path) {}
		FselList $path
		return
	}
	if [file exists $path] {
		set fsel(path) $path
		set fsel(done) 1
		return
	}
	# Neither a file or a directory.
	# See if glob will find something
	if [catch {glob $path} files] {
		# No, perhaps the user typed a new
		# absolute pathname
		if [catch {glob $fsel(path)} path] {
			# Nothing good - attempt completion
			if {$fsel(mustExist)} {
				# Attempt completion
				FselComplete
			} elseif [file isdirectory \
				[file dirname $fsel(path)]] {
				# Allow new name
				set fsel(done) 1
			}
			return
		} else {
			# OK - try again
			set fsel(dir) [file dirname $fsel(path)]
			set fsel(path) [file tail $fsel(path)]
			FselOK
			return
		}
	} else {
		# Ok - current directory is ok,
		# either select the file or list them.
		if {[llength [split $files]] == 1} {
			set fsel(path) $files
			FselOK
		} else {
			set fsel(dir) [file dirname [lindex $files 0]]
			FselList $fsel(dir) $files
		}
	}
}
proc FselCancel {} {
	global fsel
	set fsel(done) 1
	set fsel(path) {}
}

proc FselClick { lb y } {
	# Take the item the user clicked on
	global fsel
	set fsel(path) [$lb get [$lb nearest $y]]
}

proc FselTake { lb } {
	# Take the currently selected list item
	global fsel
	set fsel(path) [$lb get [$lb curselection]]
}

proc FselComplete {} {
	global fsel

	# Do file name completion
	# Nuke the space that triggered this call
	set fsel(path) [string trim $fsel(path) \t\ ]

	# Figure out what directory we are looking at
	# dir is the directory
	# tail is the partial name
	if {[string match /* $fsel(path)]} {
		set dir [file dirname $fsel(path)]
		set tail [file tail $fsel(path)]
	} elseif [string match ~* $fsel(path)] {
		if [catch {file dirname $fsel(path)} dir] {
			return	;# Bad user
		}
		set tail [file tail $fsel(path)]
	} else {
		set path $fsel(dir)/$fsel(path)
		set dir [file dirname $path]
		set tail [file tail $path]
	}
	# See what files are there
	set files [glob -nocomplain $dir/$tail*]
	if {[llength [split $files]] == 1} {
		# Matched a single file
		set fsel(dir) $dir
		set fsel(path) [file tail $files]
	} else {
		if {[llength [split $files]] > 1} {
			# Find the longest common prefix
			set l [expr [string length $tail]-1]
			set miss 0
			# Remember that files has absolute paths
			set file1 [file tail [lindex $files 0]]
			while {!$miss} {
				incr l
				if {$l == [string length $file1]} {
					# file1 is a prefix of all others
					break
				}
				set new [string range $file1 0 $l]
				foreach f $files {
					if ![string match $new* [file tail $f]] {
						set miss 1
						incr l -1
						break
					}
				}
			}
			set fsel(path) [string range $file1 0 $l]
		}
		FselList $dir $files
	}
}
#
#wm geometry . +40+40
#frame .main
#puts [Fsel_select {Select a file} {}]
#exit
