### this section is configured by the Makefile
set TkGnats(lib) /usr/lib/tkgnats; ##TKGNATSLIB##
###

foreach f { tkpr_library.tcl tkprhelp.tcl } {
    source $TkGnats(lib)/$f
}

case $TkGnats(LogName) {root} {
    Msg "You cannot edit problem reports as root.\nUse your own login"
    Exit 1
}

set CategoryList [get_categories]
if {$CategoryList == ""} {
  Msg "The categories list was empty!"
  Exit 1
}

set Tkeditpr(radioflds) {
    >State
    >Confidential
    >Severity
    >Priority
    >Class
}

proc headingMsg {a {flash 1}} {
    .mframe.msg configure -text $a
    update idletasks
    if {$flash} {
	foreach rep {1 2 3 4 5} {
	    foreach r {raised sunken flat} {
		.mframe.msg configure -relief $r;
		update idletasks
		after 50
	    }
	}
    }
}

proc Msg {a} {
    exec msgDialog [wm title .] "" $a &
    schedule_reap
}

proc chk_fld {fldname val {flag_if_missing 1}} {
    upvar 1 $fldname fld
    set mlist {}
    if {![info exists fld]} {
	if {$flag_if_missing} {
	    append mlist $fldname
	}
	set fld $val
    }
    return $mlist
}
proc load_field_defaults {field_array} {
    upvar 1 $field_array field
    chk_fld field(>State)		open
    chk_fld field(>Confidential)	no
    chk_fld field(>Severity)		serious
    chk_fld field(>Priority)		medium
    chk_fld field(>Class)		sw-bug
    chk_fld field(>Arrival-Date)	[exec date]
    chk_fld field(>Last-Modified)	None
    chk_fld field(>Originator)		Unknown
    chk_fld field(>Responsible)		gnats
    chk_fld field(>Category)		pending
    chk_fld field(>Synopsis)		None
    chk_fld field(>Release)		Unknown
    chk_fld field(>Description)		None
    chk_fld field(>Environment)		"\n"
    chk_fld field(>Audit-Trail)		"\n"
    chk_fld field(>How-To-Repeat)	"\n"

    # It's ok if these are missing
    chk_fld field(>Unformatted)		"\n" 0
    chk_fld field(>Fix)			"\n" 0
}

set Tkeditpr(singletextflds) {
    >Originator
    >Release
    >Synopsis
}

set Tkeditpr(multitextflds) {
    >Description
    >How-To-Repeat
    >Environment
    >Audit-Trail
    >Unformatted
    >Fix
}

# List of entry widgets for traverse key binding
set Tkeditpr(tlist) {}

proc merge_into_list {lname new_value} {
    upvar 1 $lname l
    if {[lsearch $l $new_value]<0} {
	set l "$l $new_value"
    }
}

proc file_report {} {
    if {[real_file_report] == -1} {
        bell
	headingMsg "Error filing report!"
	headingMsg "" 0
	return -1
    }
    return 0
}

proc real_file_report {} {
    global Tkeditpr TkGnats frm flds errorCode
    flush_singletext $Tkeditpr(singletextflds)
    flush_multitext

    #
    # do some local field checking..
    #
    headingMsg "Checking fields.." 0
    # - Check Category field...
    if {[lsearch -exact $Tkeditpr(cats) $frm(>Category)] < 0} {
        Msg "You have specified an invalid Category"
        return -1
    }
    # - Check Submitter field...
    if {[lsearch -exact $Tkeditpr(subs) $frm(>Submitter-Id)] < 0} {
        Msg "You have specified an invalid Submitter-Id"
        return -1
    }
    # - Check Responsible field...
    if {[lsearch -exact $Tkeditpr(ress) $frm(>Responsible)] < 0} {
        Msg "You have specified an invalid Responsible"
        return -1
    }
##### This doesn't work. See function in tkpr_library.tcl
#    if {"[get_passwd_entry $frm(>Responsible)]" == ""} {
#	Msg "'$frm(>Responsible)' is not a valid user"
#	return -1
#    }
    # - Check that indexed text fields do not have a | char. (hoses tkquerypr)
    foreach t {>Release >Synopsis} {
	if {[string first "|" [textget $t]] != -1} {
            Msg " '|' is an illegal character for the '$t' field"
            return -1
	}
    }
    foreach t {>Environment} {
	if {[string first "|" $frm($t)] != -1} {
            Msg " '|' is an illegal character for the '$t' field"
            return -1
	}
    }

    #
    # now see if any fields changed that trigger notifiers or audit records
    #
    # >Synopsis
    set mail_list ""
    set all_changes ""
    set datestr [exec date]

    set responsible_code     [catch {my_pr_addr $frm(>Responsible)}  responsible_addr]
    set old_responsible_code [catch {my_pr_addr $flds(>Responsible)} old_responsible_addr]
    set originator_code      [catch {my_pr_addr $frm(>Originator)}   originator_addr]
    set sender_addr $flds(Reply-To)

    if {$responsible_code} {
	set err "Could not verify E-mail address\nof new responsible party.\n"
	append err "Address used: $frm(>Responsible)\n"
	append err "Resulting error(s):\n\n$responsible_addr\n"
	set responsible_addr ""
	Msg $err
	return -1
    }
 
    if {$old_responsible_code} {
	set err "Warning: Could not verify E-mail address\n"
	append err "of old responsible party.\n"
	append err "Address used: $flds(>Responsible)\n"
	append err "Resulting error(s):\n\n$old_responsible_addr\n"
	Msg $err
	set old_responsible_addr ""
    }
 
    if {$originator_code} {
	set err "Warning: Could not verify E-mail address\n"
	append err "of Originator.\n"
	append err "Address used: $flds(>Originator)\n"
	append err "Resulting error(s):\n\n$originator_addr\n"
	Msg $err
	set originator_addr ""
    }

    set t ">Responsible"
    if {$frm($t) != $flds($t)} {

	# if we got the old_responsible persons address OK and
	# if the old responsible person is not us then mail to them
	if { ( $TkGnats(EmailAddr) != $old_responsible_addr ) &&
	     ( "$old_responsible_code" != "" )} {
	    # if they are not already in the list of course
	    merge_into_list mail_list $old_responsible_addr
	}
	# if the new responsible person is not us then mail to them
	if {$TkGnats(EmailAddr) != $responsible_addr} {
	    merge_into_list mail_list $responsible_addr
	}

	set change_msg "\n\nResponsible-Changed-From-To: $flds($t)->$frm($t)\n"
	append change_msg  "Responsible-Changed-By:      $TkGnats(LogName)\n"
	append change_msg  "Responsible-Changed-When:    $datestr\n"
	set whyText ""
	catch {
	    set whyText [entryDialog "Enter reason for Responsible change" no_cancel]
	}
	append change_msg  "Responsible-Changed-Why:     $whyText\n"

	append all_changes $change_msg 
    }
 
    set t ">State"
    if {$frm($t) != $flds($t)} {

	set change_msg	"\n\nState-Changed-From-To: $flds($t)-$frm($t)\n"
	append change_msg   "State-Changed-By:      $TkGnats(LogName)\n"
	append change_msg   "State-Changed-When:    $datestr\n"
	set whyText ""
	catch {
	    set whyText [entryDialog "Enter reason for state change" no_cancel]
	}
	append change_msg   "State-Changed-Why:     $whyText\n"

	append all_changes $change_msg 

	# if the new responsible person is not us then mail to them the status change
	if {$TkGnats(EmailAddr) != $responsible_addr} {
	    merge_into_list mail_list $responsible_addr
	}
	# if the actual sender is not us then mail to them the status change
	if {$TkGnats(EmailAddr) != $sender_addr} {
	    merge_into_list mail_list $sender_addr
	}
	# if the originator is not us then mail to them the status change
	if {$TkGnats(EmailAddr) != $originator_addr} {
	    merge_into_list mail_list $originator_addr
	}
    }

    #
    # did any notifiable changes take place ??
    #
    if {"$all_changes" != ""} {
        if {"$mail_list" != ""} {
            set   stamp  $TkGnats(LogName)[exec date +%j:%T]
            set   tmpfilename /tmp/tksendpr.$stamp
            set   fout [open $tmpfilename "w"]
            ##		set    fout [open "|$TkGnats(Mailer)" w]
            puts  $fout "To: $mail_list"
            puts  $fout "From: $TkGnats(EmailAddr)"
            puts  $fout "Reply-To: $TkGnats(EmailAddr)"
            puts  $fout "Subject: Changed information for PR $Tkeditpr(prid)"
            puts  $fout "\n\nSynopsis: $frm(>Synopsis)"
            puts  $fout $all_changes nonewline
            close $fout
            set code [catch {eval exec $TkGnats(Mailer) < $tmpfilename} result]
            exec rm $tmpfilename
        }
	append frm(>Audit-Trail) $all_changes
    }

    headingMsg "Filing report.." 0

    set stamp  $TkGnats(LogName)[exec date +%j:%T]
    set tmpfile /tmp/tksendpr.$stamp
    set fout [open $tmpfile "w"]
    write_pr $fout
    close $fout

    set errs ""
    set x [catch "exec $TkGnats(pr-edit) --check < $tmpfile" errs]
    ## "$errs" != ""
    if {0}  {
	Msg "Error checking $tmpfile:\n$errs\n$errorCode"
    } else {
	if {[catch "exec $TkGnats(pr-edit) < $tmpfile" errs]} {
	    Msg "Error filing $tmpfile:\n$errs"
	} else {
	    headingMsg "Done" 0
	    exec rm -f $tmpfile
	    Exit 0
	}
    }
    exec rm -f $tmpfile
    return -1
}

proc write_multitextfld {fout flds tag} {
    upvar 1 $flds f
    puts $fout [string trimleft "$tag: \n[string trim $f($tag) "\n"]" "\n"]
}

proc write_pr {fout} {
    global Tkeditpr flds frm

    foreach tag [array names frm] {
	set still_left($tag) $tag
    }

    #
    # for each parsed field from the PR form...
    #
    foreach tag $Tkeditpr(parsed_flds) {
	case $tag {_prefix_} {
	    #
	    #	The mail header, stored under the _prefix_ tag, is written out
	    #	unadulterated.
	    #
	    puts $fout $flds($tag) nonewline
	} {>Unformatted} {
	    #
	    # Taken care of later in the function..
	    #
	} {>*} {
	    # When writing out the fields
	    #	first check for data present in the form (the frm bag)
	    #	If not present use data read from the PR file (the flds bag)
	    #
	    if {[info exists frm($tag)]} {
		set data $frm($tag)
		unset still_left($tag)
	    } else {
		set data $flds($tag)
	    }

	    #
	    # Write out fields
	    #
	    #	Multi line fields are newline trimmed to a single leading
	    #	and trailing newline
	    #
	    #	Single line text fields are whitspace trimed to a leading
	    #	tab and a trailing newline
	    #
	    case $tag $Tkeditpr(singletextflds) {
		puts $fout "$tag:\t[string trim [textget $tag] "\t\n "]"
	    } [concat >Category >Responsible >Submitter-Id $Tkeditpr(radioflds)] {
		puts $fout "$tag:\t$frm($tag)"
	    } $Tkeditpr(multitextflds) {
		write_multitextfld $fout frm $tag
	    } default {
		puts $fout "$tag:$data" nonewline
	    }
	}
    }

    #
    # now write any fields in the form that were not in the parsed report
    #
    foreach tag [array names still_left] {
	write_multitextfld $fout frm $tag
    }

    #
    # Finally, write the >Unformatted field
    #   (BUG: >Unformatted should not really be stripped)
    #
    write_multitextfld $fout frm ">Unformatted"
}



# bail out completely
proc cancel_report {} {
    Exit 0
}

proc edit_category_listbox {p {pat *} {valwid 0}} {
    global flds frm Tkeditpr
    set Tkeditpr(cats) [get_categories $pat]
    if {$valwid == 0} {
        set valwid [string length [ftrim $flds(>Category)]]
    }

    set wid [expr 2 + [get_max_strlen $Tkeditpr(cats)]]
    set Tkeditpr(listbox_width) $wid
    
    frame  $p.cat -relief flat
    pack   $p.cat -side top -anchor w
    button $p.cat.lab -text "Category: " -width 14 -anchor w -command "helpMsg Category" \
            -relief flat -padx 0 -pady 0 -borderwidth 0 -highlightthickness 1
    frame  $p.cat.msg -relief flat
    label  $p.cat.msg.val -text "[ftrim $flds(>Category)]" -relief groove -anchor w \
            -width $wid -background green
    set ew [entry $p.cat.msg.ent -width $wid -insertwidth 1 -insertofftime 400 \
            -relief sunken -borderwidth 2 -background grey95 -textvariable frm(>Category)]
    lappend Tkeditpr(tlist) $ew
    bind $ew <Enter> "+focus $ew"
    scrollbar $p.cat.msg.sb -command "$p.cat.msg.list yview" -borderwidth 2 -relief sunken
    set height [llength $Tkeditpr(cats)]
    if {$height > 6 } {
        set height 6
    }
    set Tkeditpr(listbox_height) $height
    listbox $p.cat.msg.list -yscroll "$p.cat.msg.sb set" -setgrid 1 -relief sunken -borderwidth 2 \
            -width $wid -height $height -exportselection false
    pack $p.cat.msg.val  -side top   -fill both -expand true -anchor center
    pack $p.cat.msg.ent  -side top   -fill both -expand true -anchor w
    pack $p.cat.msg.list -side left  -fill both -expand true
    pack $p.cat.msg.sb   -side right -fill y
    eval $p.cat.msg.list insert end $Tkeditpr(cats)
    trace variable frm(>Category) w set_edit_category_ew
    pack $p.cat.lab $p.cat.msg -side left -anchor n
    bind   $p.cat.msg.list <B1-ButtonRelease> "set_edit_category $p.cat.msg %W %y"
    return $p.cat.msg.list
}

proc edit_responsible_listbox {p {pat *} {valwid 0}} {
    global flds frm Tkeditpr
    set Tkeditpr(ress) [get_responsibles $pat]
    if {$valwid == 0} {
        set valwid [string length [ftrim $flds(>Responsible)]]
    }

    set wid [expr 2 + [get_max_strlen $Tkeditpr(ress)]]
    frame  $p.res -relief flat
    pack   $p.res -side top -anchor w
    button $p.res.lab -text "Responsible: " -width 14 -anchor w -command "helpMsg Responsible" \
            -relief flat -padx 0 -pady 0 -borderwidth 0 -highlightthickness 1
    frame  $p.res.msg -relief flat
    label  $p.res.msg.val -text "[ftrim $flds(>Responsible)]" -relief groove -anchor w \
            -width $wid -background green
    set ew [entry $p.res.msg.ent -width $wid -insertwidth 1 -insertofftime 400 \
            -relief sunken -borderwidth 2 -background grey95 -textvariable frm(>Responsible)]
    lappend Tkeditpr(tlist) $ew
    bind $ew <Enter> "+focus $ew"
    scrollbar $p.res.msg.sb -command "$p.res.msg.list yview" -borderwidth 2 -relief sunken
    set height [llength $Tkeditpr(ress)]
    if {$height > 6 } {
        set height 6
    }
    listbox $p.res.msg.list -yscroll "$p.res.msg.sb set" -setgrid 1 -relief sunken \
            -borderwidth 2 -width $wid -height $Tkeditpr(listbox_height) -exportselection false
    pack $p.res.msg.val  -side top   -fill both -expand true -anchor center
    pack $p.res.msg.ent  -side top   -fill both -expand true -anchor w
    pack $p.res.msg.list -side left  -fill both -expand true
    pack $p.res.msg.sb   -side right -fill y
    eval $p.res.msg.list insert end $Tkeditpr(ress)
    trace variable frm(>Responsible) w set_edit_responsible_ew
    pack $p.res.lab $p.res.msg -side left -anchor n
    bind   $p.res.msg.list <B1-ButtonRelease> "set_edit_responsible $p.res.msg %W %y"
    return $p.res.msg.list
}

proc edit_submitter_listbox {p {pat *} {valwid 0}} {
    global flds frm Tkeditpr
    set Tkeditpr(subs) [get_submitters $pat]
    if {$valwid == 0} {
        set valwid [string length [ftrim $flds(>Submitter-Id)]]
    }

    set wid [expr 2 + [get_max_strlen $Tkeditpr(subs)]]
    frame  $p.sub -relief flat
    pack   $p.sub -side top -anchor w
    button $p.sub.lab -text "Submitter-Id: " -width 14 -anchor w -command "helpMsg Submitter-Id" \
            -relief flat -padx 0 -pady 0 -borderwidth 0 -highlightthickness 1
    frame  $p.sub.msg -relief flat
    label  $p.sub.msg.val -text "[ftrim $flds(>Submitter-Id)]" -relief groove -anchor w \
            -width $wid -background green
    set ew [entry $p.sub.msg.ent -width $wid -insertwidth 1 -insertofftime 400 \
            -relief sunken -borderwidth 2 -background grey95 -textvariable frm(>Submitter-Id)]
    lappend Tkeditpr(tlist) $ew
    bind $ew <Enter> "+focus $ew"
    scrollbar $p.sub.msg.sb -command "$p.sub.msg.list yview" -borderwidth 2 -relief sunken
    set height [llength $Tkeditpr(subs)]
    if {$height > 6 } {
        set height 6
    }
    listbox $p.sub.msg.list -yscroll "$p.sub.msg.sb set" -setgrid 1 -relief sunken \
            -borderwidth 2 -width $wid -height $Tkeditpr(listbox_height) -exportselection false
    pack $p.sub.msg.val  -side top   -fill both -expand true -anchor center
    pack $p.sub.msg.ent  -side top   -fill both -expand true -anchor w
    pack $p.sub.msg.list -side left  -fill both -expand true
    pack $p.sub.msg.sb   -side right -fill y
    eval $p.sub.msg.list insert end $Tkeditpr(subs)
    trace variable frm(>Submitter-Id) w set_edit_submitter_ew
    pack $p.sub.lab $p.sub.msg -side left -anchor n
    bind   $p.sub.msg.list <B1-ButtonRelease> "set_edit_submitter $p.sub.msg %W %y"
    return $p.sub.msg.list
}

proc set_edit_category_ew {a b c} {
    upvar #0 $a f
    global Tkeditpr
    quickfill_entry_from_listbox f($b) .eboxs.clb.cat.msg.ent .eboxs.clb.cat.msg.list $Tkeditpr(cats)
}

proc set_edit_submitter_ew {a b c} {
    upvar #0 $a f
    global Tkeditpr
    quickfill_entry_from_listbox f($b) .eboxs.slb.sub.msg.ent .eboxs.slb.sub.msg.list $Tkeditpr(subs)
}

proc set_edit_responsible_ew {a b c} {
    upvar #0 $a f
    global Tkeditpr
    quickfill_entry_from_listbox f($b) .eboxs.rlb.res.msg.ent .eboxs.rlb.res.msg.list $Tkeditpr(ress)
}

proc set_edit_category {msg w y} {
    global frm
    trace vdelete  frm(>Category) w set_edit_category_ew
    $msg.ent delete 0 end
    set idx [$w nearest $y]
    set frm(>Category) [$w get $idx]
    trace variable frm(>Category) w set_edit_category_ew
}

proc set_edit_responsible {msg w y} {
    global frm
    trace vdelete  frm(>Responsible) w set_edit_responsible_ew
    $msg.ent delete 0 end
    set idx [$w nearest $y]
    set frm(>Responsible) [$w get $idx]
    trace variable frm(>Responsible) w set_edit_responsible_ew
}

proc set_edit_submitter {msg w y} {
    global frm
    trace vdelete  frm(>Submitter) w set_edit_submitter_ew
    $msg.ent delete 0 end
    set idx [$w nearest $y]
    set frm(>Submitter-Id) [$w get $idx]
    trace variable frm(>Submitter) w set_edit_submitter_ew
}

set current_multi_text ""

proc sanity_check {fields} {
}

set Tkeditpr(prid) [lindex $argv 0]
if {$Tkeditpr(prid) == ""} {
    Msg "missing prid argument"
    Exit -1
}

######set Tkeditpr(srcfile) [lindex $argv 1]
######if {"$Tkeditpr(srcfile)" == ""} {
######    Msg "missing srcfile argument"
######    Exit -1
######}

# load a bunch of defaults into flds.

set fin [open "|query-pr --full [file tail $Tkeditpr(prid)]" r]
set Tkeditpr(parsed_flds) [parsepr $fin flds]
set missing_list [load_field_defaults flds]
close $fin
if {"$missing_list" != ""} {
    Msg "The following fields were missing in the PR\n$missing_list"
    set Tkeditpr(parsed_flds) [concat $Tkeditpr(parsed_fields) $missing_list]
}

frame   .mframe     -borderwidth 1 -relief raised
pack    .mframe     -side top -fill x
message .mframe.msg -aspect 10000 
pack    .mframe.msg -side left -fill x

menubutton .mframe.help -text "Help" -menu .mframe.help.m -underline 0
menu       .mframe.help.m
.mframe.help.m add command -label "Overview" \
        -command "helpMsg Edit_Overview"
.mframe.help.m add separator
.mframe.help.m add command -label "Field Definitions" \
        -command "helpMsg Field_Definitions"
.mframe.help.m add separator
.mframe.help.m add command -label "Radio Buttons (Class, etc)" \
        -command "helpMsg Edit_Radio_Buttons"
.mframe.help.m add command -label "Listbox Selectors (Category, etc)" \
        -command "helpMsg Edit_Listbox_Selectors"
.mframe.help.m add command -label "Entry Fields (Originator, etc)" \
        -command "helpMsg Edit_Entry_Fields"
.mframe.help.m add command -label "Text Fields (Description, etc)" \
        -command "helpMsg Edit_Text_Fields"
pack       .mframe.help -side right

# Get the maximum width of the value fields for the bagged_radiobar
set Tkeditpr(value_width) 0
foreach tag [concat >Category >Responsible $Tkeditpr(radioflds)] {
    set f [ftrim $flds($tag)]
    if {[string length $f] > $Tkeditpr(value_width)} {
        set Tkeditpr(value_width) [string length $f]
    }
}

# Get the maximum width of the value fields for the readonly_singletext fields
###set Tkeditpr(singletextvalue_width) \
###        [string length "[ftrim $flds(>Originator)] (email: $flds(Reply-To))"]
set Tkeditpr(singletextvalue_width) 0
foreach tag [concat >Last-Modified >Arrival-Date] {
    set f [ftrim $flds($tag)]
    if {[string length $f] > $Tkeditpr(singletextvalue_width)} {
        set Tkeditpr(singletextvalue_width) [string length $f]
    }
}

#####readonly_singletext Submitter-Id  [ftrim $flds(>Submitter-Id)]  14 \
#####        $Tkeditpr(singletextvalue_width)
###readonly_singletext Originator   "[ftrim $flds(>Originator)] (email: $flds(Reply-To))"  14 \
###        $Tkeditpr(singletextvalue_width)
readonly_singletext Arrival-Date  [ftrim $flds(>Arrival-Date)]  14 \
        $Tkeditpr(singletextvalue_width)
readonly_singletext Last-Modified [ftrim $flds(>Last-Modified)] 14 \
        $Tkeditpr(singletextvalue_width)

frame .eflds
radiobar_frame .eflds .eflds.lb
bagged_radiobar .eflds.lb class Class \
    {sw-bug doc-bug change-request support mistaken duplicate} None frm $Tkeditpr(value_width)
bagged_radiobar .eflds.lb state "State" \
    {open analyzed feedback closed suspended} None frm $Tkeditpr(value_width)
bagged_radiobar .eflds.lb priority Priority \
    {low medium high} None frm $Tkeditpr(value_width)
bagged_radiobar .eflds.lb severity Severity \
    {non-critical serious critical} None frm $Tkeditpr(value_width)
bagged_radiobar .eflds.lb confidential "Confidential" \
    {no yes} None frm $Tkeditpr(value_width)
pack .eflds.lb -side left -pady 0
pack .eflds    -side top  -pady 0 -fill x -anchor w

frame .eboxs
frame .eboxs.clb -relief groove -borderwidth 2
edit_category_listbox .eboxs.clb * $Tkeditpr(value_width)
frame .eboxs.slb -relief groove -borderwidth 2
edit_submitter_listbox   .eboxs.slb * $Tkeditpr(value_width)
frame .eboxs.rlb -relief groove -borderwidth 2
edit_responsible_listbox .eboxs.rlb * $Tkeditpr(value_width)

pack .eboxs.clb -side left -anchor w -pady 2 -padx 0
pack .eboxs.slb -side left -anchor w -pady 2 -padx 4
pack .eboxs.rlb -side left -anchor w -pady 2 -padx 0
pack .eboxs -side top -fill x -anchor w -pady 2 -padx 0

lappend Tkeditpr(tlist) [singletext . >Originator  80 "" 14]
lappend Tkeditpr(tlist) [singletext . >Release     80 "" 14]
lappend Tkeditpr(tlist) [singletext . >Synopsis    80 "" 14]

lappend Tkeditpr(tlist) [make_txt_mb $Tkeditpr(multitextflds)]

set_text_traversal $Tkeditpr(tlist)

frame  .action -borderwidth 3
button .action.send   -borderwidth 2 -text "Save Changes"              -command file_report
button .action.cancel -borderwidth 2 -text "Cancel"                    -command cancel_report
button .action.reset  -borderwidth 2 -text "Reset to Starting Values"  -command reset_report
button .action.email  -borderwidth 2 -text "Send Email..." \
	-command "email_originator [ftrim $flds(Reply-To)] [ftrim $flds(>Responsible)] \
        [lrange [split [ftrim $flds(>Originator)]] 0 0] \
        [ftrim $Tkeditpr(prid)] [list [ftrim $flds(>Synopsis)]]"

pack .action.send .action.reset .action.email .action.cancel -side left -padx 8
pack .action -side top

proc fillfrm {} {
    global frm flds Tkeditpr

    ### pre-set editable PR values to values currently in the PR

    # clear listbox entry widgets
    .eboxs.clb.cat.msg.ent delete 0 end
    .eboxs.slb.sub.msg.ent delete 0 end
    .eboxs.rlb.res.msg.ent delete 0 end

    # listboxes and radio (enumerated) fields
    foreach tag [concat >Category >Responsible >Submitter-Id $Tkeditpr(radioflds)] {
	set flds($tag) [string trim $flds($tag) "\t\n "]
	set frm($tag) $flds($tag)
    }

    # now the 1 line textual flds
    foreach tag $Tkeditpr(singletextflds) {
	set flds($tag) [string trim $flds($tag) "\t\n "]
	textset $tag $flds($tag)
    }

    # now the multi line textual flds
    foreach tag $Tkeditpr(multitextflds) {
	if {[info exists flds($tag)]} {
	    set $flds($tag) [string trim $flds($tag) "\n"]
	    set frm($tag) $flds($tag)
	} else {
	    set frm($tag)  "\n"
	    set flds($tag) "\n"
	}
    }
    switch_txt ">Description" $Tkeditpr(multitextflds)
}

proc reset_report {} {
    global flds current_multi_text Tkeditpr
    # load the current text widget with the original text
    .multiline.text delete 1.0 end
    .multiline.text insert 1.0 $flds($current_multi_text)
    # reset everything
    set current $current_multi_text
    fillfrm
    switch_txt $current $Tkeditpr(multitextflds)
}

wm title      . "TkGnats - Edit Problem Report: [ftrim $flds(>Category)]/[ftrim $flds(>Number)]"
wm iconbitmap . @$TkGnats(lib)/tkeditpr.xbm
wm iconname   . "$env(LOGNAME)'s tkeditpr [ftrim $flds(>Number)]"
fillfrm
