#
# $Id: tkpr_library.t,v 1.3 1993/11/16 22:19:36 jason Exp $
#

set TkGnats(lib) /usr/lib/tkgnats; ##TKGNATSLIB##
source $TkGnats(lib)/tkgnats.cf.t

if {[file readable $TkGnats(lib)/tkgnatsrc]} {
    source $TkGnats(lib)/tkgnatsrc
}
if {[file readable ~/TkGnats/tkgnatsrc]} {
    source ~/TkGnats/tkgnatsrc
}

# Determine if this user is authorized to edit problem reports
set glist {}
set ulist {}
if {[info exists TkGnats(edit_authorized_groups)]} {
    set glist $TkGnats(edit_authorized_groups)
}
if {[info exists TkGnats(edit_authorized_users)]} {
    set ulist $TkGnats(edit_authorized_users)
}
set TkGnats(edit_authorized) 0
if {$ulist == {} && $glist == {}} {
    set TkGnats(edit_authorized) 1
}
if {$ulist != {} && [lsearch -exact $ulist $TkGnats(LogName)] > -1} {
    set TkGnats(edit_authorized) 1
}
if {$glist != {} && [lsearch -exact $glist $TkGnats(GroupName)] > -1} {
    set TkGnats(edit_authorized) 1
}

#
proc Msg {a {flash 1}} {
    puts stderr $a
}

# reap any zombied exec's
set TkGnats(reap_scheduled) 0
proc do_reap {} {
    global TkGnats
    exec true
    set TkGnats(reap_scheduled) 0
}
proc schedule_reap {} {
    global TkGnats
    if {!$TkGnats(reap_scheduled)} {
	set TkGnats(reap_scheduled) 1
	after 5000 do_reap
    }
}

proc Exit {x} {
    #destroy .; # use if before tk3.3
    exit $x
}

proc my_pr_addr {name} {
    global TkGnats
    # strip off whitespace and then take out the first word.
    # we assume the rest is a (Full Name) type comment
    set name [string trim $name "\t\n "]
    set name [lindex [split $name "\t\n "] 0]
    set tmp [exec $TkGnats(pr-addr) $name]
    if {"$tmp" != ""} {
	set name $tmp
    }
    return $name
}

proc check_tkgnats_userdir {} {
    if {![file isdirectory [glob ~]/TkGnats]} {
        exec mkdir [glob ~]/TkGnats
    }
    if {![file isdirectory [glob ~]/TkGnats/query]} {
        exec mkdir [glob ~]/TkGnats/query
    }
    if {![file isdirectory [glob ~]/TkGnats/sort]} {
        exec mkdir [glob ~]/TkGnats/sort
    }
}

#
# trim pr field data whitespace
#
proc ftrim {s} {
    return [string trim $s "\t\n "]
}

#
# get a list of valid gnats categories
#
proc get_categories {{pat "*"} {file ""}} {
    global TkGnats LIBDIR
    if {"$file" == ""} {
#	set file $LIBDIR/gnats/$TkGnats(GNATS_SITE)
#	set file $GNATS_BASE/$TkGnats(GNATS_SITE)
	set file $TkGnats(CategoriesFile)
    }
    set catlist {}
    if {[file readable $file]} {
	foreach c [split [exec cat $file] "\n"] {
	    # ignore lines with leading hash or underscore
	    case $c "#*" {
	    } "_*" {
	    } $pat {
		lappend catlist [lindex [string trim $c "\n\t "] 0]
	    }
	}
    } else {
	Msg "Cannot read category file $file"
	return ""
    }
    return [lsort $catlist]
}

#
# get a list of valid gnats submitters
#
proc get_submitters {{pat "*"} {file ""}} {
    global TkGnats
    if {"$file" == ""} {
	set file $TkGnats(SubmittersFile)
    }
    set sublist {}
    if {[file readable $file]} {
	foreach s [split [exec cat $file] "\n"] {
	    # ignore lines with leading hash or underscore
	    case $s "#*" {
	    } "_*" {
	    } $pat {
		lappend sublist [lindex [split $s ":"] 0]
	    }
	}
    } else {
	Msg "Cannot read submitters file $file"
	return ""
    }
    return [lsort $sublist]
}

#
# get a list of valid gnats responsibles
#
proc get_responsibles {{pat "*"} {file ""}} {
    global TkGnats
    if {"$file" == ""} {
	set file $TkGnats(ResponsibleFile)
    }
    set reslist {}
    if {[file readable $file]} {
	foreach r [split [exec cat $file] "\n"] {
	    # ignore lines with leading hash or underscore
	    case $r "#*" {
	    } "_*" {
	    } $pat {
		lappend reslist [lindex [split $r ":"] 0]
	    }
	}
    } else {
	Msg "Cannot read responsible file $file"
	return ""
    }
    return [lsort $reslist]
}

#
# get the users name from the passwd file given their logname
# try NIS first, then try the regular passwd file
#
proc fullname_from_logname {{lname ""}} {
    global TkGnats
    if {"$lname" == ""} {
	set lname $TkGnats(LogName)
    }
    if {
     [catch {set fullname [exec ypcat passwd | grep ^${lname}: | cut -f5 -d:]}]
    } {
	set fullname [exec cat /etc/passwd | grep ^${lname}: | cut -f5 -d:]
    }
    return $fullname
}

proc get_passwd_entry {lname} {
    # on RickM's linux machine, without NIS, this returns: "child process exited abnormally"
    # IE it doesn't fall through to the regular passwd file.
    
    # first try NIS, then try the regular passwd file..
    if {
     [catch {set fullname [exec ypcat passwd | grep ^${lname}:]}]
    } {
	catch "exec cat /etc/passwd | grep ^${lname}:" fullname
    }
    return $fullname
}

proc radiobar_frame {parent frname} {
    frame $frname
    frame $frname.labels
    frame $frname.values
    frame $frname.bars
    pack  $frname.labels -side left  -anchor w
    pack  $frname.values -side left  -anchor center -padx 2
    pack  $frname.bars   -side right -expand true   -fill x
    pack  $frname -anchor w
}

# text field related procs

proc textset {l t {p ""}} {
    # a label
    if {[winfo exists ${p}._${l}.textlabel]} {
	${p}._${l}.textlabel configure -text $t
	return
    }
    # text widget
    if {[winfo exists ${p}._${l}_fr.text]} {
	${p}._${l}_fr.text delete 1.0 end
	${p}._${l}_fr.text insert 1.0 $t
	return
    }
    # entry widget
    if {[winfo exists ${p}._${l}.text]} { 
	${p}._${l}.text delete 0 end
	${p}._${l}.text insert 0 $t
	return
    }
    error "no such window for $l"
}

proc textget {l {p ""}} {
    if {[catch  {set x [lindex [${p}._${l}.textlabel configure -text] 4]}  ]} {
	if {[catch {set x [${p}._${l}_fr.text get 1.0 end]}]} {
	    return [string trim [${p}._${l}.text get] "\n"]
	}
    }
    return "\n$x"
}

proc readonly_singletext {l {t ""} {labwid 12} {valwid 0}} {
    if {$valwid == 0} {
        set valwid [string length "$t"]
    }
    set f  [frame ._${l}]
    set lw [button $f.label    -anchor w -width $labwid -text "$l: " -command "helpMsg $l" \
            -relief flat -padx 0 -pady 0 -borderwidth 0 -highlightthickness 1]
    set ew [label $f.textlabel -anchor w -width $valwid -text "$t" -relief groove \
            -background green -padx 2 -pady 0]
    pack $lw -side left -anchor w -pady 0 -padx 1
    pack $ew -side left -anchor w -pady 0
    pack $f  -side top  -anchor w -pady 0 -fill x
    return $ew
}

bind Entry <KeyPress-Return> " "

proc singletext {parent l w {t ""} {labwid 12}} {
    set f [frame $parent._${l}]
    # trim off any leading >'s for the label text
    set lw [button $f.label -anchor w -width $labwid -text "[string trimleft $l >]: " \
            -command "helpMsg [string trimleft $l >]" -relief flat -padx 0 -pady 0 -borderwidth 0]
    set ew [entry $f.text -width $w \
	-insertwidth 1	-insertofftime 400  \
	-relief sunken -borderwidth 2 -background grey95]
    bind $ew <Enter> "+focus $ew"
    $ew insert end $t
    pack $lw -side left -anchor w
    pack $ew -side left -anchor w -fill x -expand true
    pack $f  -side top  -anchor w -fill x -pady 2
    return $ew
}

proc bagged_singletext {l w bagname {prefix ""} {t ""}} {
    upvar #0 $bagname bag
    set f [frame ._${l}]
    set lw [label $f.label -anchor w -text "$l: "]
    set ew [entry $f.text -width 80 \
	-insertwidth 1	-insertofftime 400  \
	-relief sunken -borderwidth 2 -background grey95]
    bind $ew <Enter> "+focus $ew"
    $ew insert end $t
    pack $lw -side left  -anchor w -fill x -expand true
    pack $ew -side right -anchor e
    pack $f  -side top   -anchor w -fill x -pady 2
    set bag($prefix$l) [format {[string trim [%s get]]} $lw]
    return $ew
}

proc multitextdeck {w blist h} {
    global TkGnats
    frame $w 
    menubutton $w.mb
    text $f.text \
	-wrap $TkGnats(TextWrap) \
	-yscrollcommand "$f.sb set" \
	-height $h -width 80 -relief sunken -padx 4 -insertwidth 1 \
	-insertofftime 400 -borderwidth 2 -background grey95
    foreach lbl $blist {
	set l _$lbl
    }
}

proc multitext {lbl h} {
    global TkGnats
    set l _$lbl
    set f [frame .${l}_fr]
    button $f.label -anchor w -text "[string trimleft $lbl >]: " \
            -command "helpMsg [string trimleft $lbl >]" \
            -relief flat -padx 0 -pady 0 -borderwidth 0
    text $f.text \
	-wrap $TkGnats(TextWrap) \
	-yscrollcommand "$f.sb set" \
	-height $h -width 80 -relief sunken -padx 4 -insertwidth 1 \
	-insertofftime 400 -borderwidth 2 -background grey95
    bind $f.text <Enter> "+focus $f.text"
    scrollbar $f.sb -command "$f.text yview" -relief sunken
    pack $f.label -side top   -anchor w
    pack $f.sb    -side left  -fill y
    pack $f.text  -side right -fill both -expand true
    pack $f       -side top   -fill both -expand true -padx 16 -pady 2

    return $f.text
}

proc set_text_traversal {tlist} {
    set ll [llength $tlist]
    if {$ll < 2} {
	return
    }
    for {set x 1} {$x<$ll} {incr x} {
	set w [lindex $tlist $x]
	set prevw [lindex $tlist [expr $x-1]]
	bind $prevw <Tab>       "focus $w
        break"
	bind $prevw <Control-n> "focus $w"
    }
    bind [lindex $tlist [expr $ll-1]] <Tab>       "focus [lindex $tlist 0]
    break"
    bind [lindex $tlist [expr $ll-1]] <Control-n> "focus [lindex $tlist 0]"
    
    bind [lindex $tlist 0] <Shift-Tab> "focus [lindex $tlist [expr $ll-1]]
        break"
    bind [lindex $tlist 0] <Control-p> "focus [lindex $tlist [expr $ll-1]]"
    for {set x 0} {$x < [expr $ll-1]} {incr x} {
	set w [lindex $tlist $x]
	set nextw [lindex $tlist [expr $x+1]]
	bind $nextw <Shift-Tab> "focus $w
        break"
	bind $nextw <Control-p> "focus $w"
    }
}

proc bagged_radiobar {fr n labeltext blist offLabel dstbag {valwid 0}} {
    radiobar $fr $n $labeltext $blist $offLabel > $dstbag $valwid
}

# make one in a list a radiobutton bar
proc radiobar {fr n labeltext blist offLabel {varprefix ""} {aname ""} {valwid 0}} {
    global flds
    if {"$aname" != ""} {
	set vname [set aname]($varprefix$labeltext)
    } else {
	set vname $varprefix$labeltext
    }
    global $vname
    
    set $vname ""
    button $fr.labels.$n -text "${labeltext}: " -command "helpMsg $labeltext" \
            -relief flat -padx 0 -pady 0 -borderwidth 0 -width 14 -anchor w -highlightthickness 1
    pack   $fr.labels.$n -side top -pady 0 -padx 0 -anchor w
    
    if {$valwid != 0} {
        label $fr.values.$n -text "[string trim $flds($varprefix$labeltext) " \n\t"]" \
                -relief groove -anchor w -width $valwid -background green -padx 2 -pady 0
        pack  $fr.values.$n -side top -expand true -fill x -pady 0 -padx 0 -anchor w
    }

    frame $fr.bars.$n
    foreach b $blist {
	radiobutton $fr.bars.$n._$b \
		-text $b -relief flat -variable $vname -pady 0 -highlightthickness 0
	# Buttons that say None should set variable to the empty
	# string...
	if {"$b" == "$offLabel"} {
	    $fr.bars.$n._$b configure -value ""
	} else {
	    $fr.bars.$n._$b configure -value $b
	}
	pack $fr.bars.$n._$b -side left -padx 8 -pady 0 -anchor w -ipadx 0
    }
    pack $fr.bars.$n -side top -expand true -fill x -padx 0 -pady 0 -anchor w
}

proc radiobar_set {fr n b} {
    $fr.bars.$n._$b invoke
}

# make one in a list a radiobutton bar
proc checkbar {fr n labeltext blist offLabel} {
    upvar #0 gbag ${labeltext}
    button $fr.labels.$n -text "${labeltext}: " -command "helpMsg $labeltext" \
            -relief flat -width 14 -padx 0 -pady 0 -borderwidth 0 -anchor w -highlightthickness 1
    pack   $fr.labels.$n -side top -pady 0 -anchor w
    frame  $fr.bars.$n
    foreach b $blist {
	checkbutton $fr.bars.$n._$b \
		-offvalue "" \
		-text $b -relief flat -highlightthickness 0 \
		-variable [format "%s(%s)" ${labeltext} ${b}] -pady 0
	# Buttons that say None should set variable to the empty
	# string...
	if {"$b" == "$offLabel"} {
	    $fr.bars.$n._$b configure -onvalue "_ALL_" -offvalue ""
	} else {
	    $fr.bars.$n._$b configure -onvalue $b -offvalue ""
	}
	set gbag($b) ""
	pack $fr.bars.$n._$b -side left -padx 8 -pady 0 -anchor w
    }
    # set active [lindex $blist 0]
    # $fr.bars.$n.$active select
    pack $fr.bars.$n -side top -expand true -fill x -pady 0 -anchor w
}

#
# convert some numeric fields in a 'query-pr --sql'
# record named 'f' in the caller to mnemonic strings
#
proc convertsqlflds {f} {
    upvar 1 $f flds
    foreach a [array names flds] {
	set n $flds($a)
	case $a Severity {
	    case $n 1 { } 2 { } 3 { }
	} Priority {
	    case $n 1 { } 2 { } 3 { }
	}
    }
}

#
# split a pr stream into a tcl array named v
#
# A special array index called _prefix_ contains  all the text prior to
# to the first gnats field
#
proc parsepr {fin varname} {
    upvar 1 $varname fields
    set gnats_tag_exp {^(>[^:]+):(.*)}
    set mail_tag_exp {^([A-Z][^:]+):[ 	]+(.*)}
    set no_gnats_tags_yet 1
    set fields(_prefix_) ""
    set fldtags {_prefix_}

    set leftoverln ""
    while {1} {

	if {"$leftoverln" == ""} {
	    set x [gets $fin ln]
	    if {$x < 0} {
		break
	    }
	} else {
	    set ln $leftoverln
	    set leftoverln ""
	}

	set tag ""
	set val ""

	regexp $gnats_tag_exp $ln matched tag val
	if {"$tag" != ""} {
	    set no_gnats_tags_yet 0
	    # a gnats tag
	    # gnats tags can bu multiline so now
	    # get all the lines 'till the next gnats tag
	    lappend fldtags $tag
	    set fields($tag) "$val\n"
	    while {[gets $fin ln]>=0} {
		set tag2 ""
		regexp $gnats_tag_exp $ln matched tag2 val
		if {"$tag2" != ""} {
		    #  a new gnats tag so we have hit the end of the 
		    # current one.. leave the line we just read in
		    # leftoverln and continue on in the loop
		    set leftoverln $ln
		    break;
		}
		append fields($tag) "$ln\n"
	    }
	    continue
	}
	if {$no_gnats_tags_yet} {
	    append fields(_prefix_) "$ln\n"
	}

	# If we get here the current line is not part of a gnats tag
	# value pair
	set tag ""
	set val ""
	# Here is where we split out regular mail headers if needed.
	regexp $mail_tag_exp $ln matched tag val
	if {"$tag" != ""} {
	    lappend fldtags $tag
	    set fields($tag) "$val\n"
	}
    }

    #
    # Do a little post processing before on the fields we leave
    #
    # For the Reply-To: field make sure the (descriptive name) part of
    #	logname (descriptive name)
    # for the email adress is stripped out
    #
    if {![info exists fields(Reply-To)]} {
	if {[info exists fields(Sender)]} {
	    set fields(Reply-To) $fields(Sender)
	} else {
	    if {[info exists fields(From)]} {
		set fields(Reply-To) $fields(From)
	    } else {
		set fields(Reply-To) "_unknown_"
	    }
	}
    }
    set fields(Reply-To) [lindex [string trim $fields(Reply-To) " \n\t"] 0]
    return $fldtags
}

proc write_listbox {lbname fname} {
    global Query
    set fout [open $fname w]
    set sz [$lbname size]
    for {set x 0} {$x < $sz} {incr x 1} {
	puts $fout [$lbname get $x]
    }
    close $fout
}

proc write_listbox_selection {lbname fname} {
    global Query
    set    fout [open $fname w]
    puts  $fout [$lbname get [$lbname curselection]]
    close $fout
}

proc foreach_listbox {lbname procname} {
    set sz [$lbname size]
    for {set x 0} {$x < $sz} {incr x 1} {
	if {[$procname [$lbname get $x]] != 0} {
	    return
	}
    }
}

proc get_max_strlen { l } {
    set maxlen 0
    foreach e $l {
        if {[string length $e] > $maxlen} {
            set maxlen [string length $e]
        }
    }
    return $maxlen
}

proc build_sort_cmd {fieldnames fieldflgs sortfields} {
    set sz [llength $sortfields]
    set rval "sort -fb -t|"
    for {set x 0} {$x < $sz} {incr x 1} {
	set fname [lindex $sortfields $x]
	set idx [lsearch $fieldnames $fname]
	if {[llength $fieldflgs] > $idx} {
	    set flgs [lindex $fieldflgs $idx]
	} else {
	    set flgs ""
	}
	append rval [format " +%d%s -%d" $idx $flgs [expr $idx+1]]
    }
    return "$rval"
}

proc sort_listbox {lb} {
    set vals [lsort [$lb get 0 end]]
    $lb delete 0 end
    eval $lb insert end $vals
}

proc make_txt_mb {multitextflds} {
    global TkGnats
    set   f [frame .multiline -relief groove  -borderwidth 2]
    
    frame  .mb
    button .mb.lab -text "Text Fields =>" -width 14 -anchor w -command "helpMsg Text-Fields" \
            -relief flat -padx 0 -pady 0 -borderwidth 0
    pack   .mb.lab -side left -anchor w
    set num 0
    foreach x $multitextflds {
	set lbl [string trimleft $x >]
	button .mb.b$num -text "${lbl}:" -command "switch_txt $x {$multitextflds}"
        pack   .mb.b$num -side left -anchor w
        incr num
    }
    button .mb.insert -text "Insert File..." -command "insert_file .multiline.text"
    pack   .mb.insert -side right -anchor e

    pack   .mb -side top -anchor w -fill x -in $f

    text $f.text -height 12 -width 80 -relief sunken -padx 4 -insertwidth 1 \
            -wrap $TkGnats(TextWrap) -yscrollcommand "$f.sby set" -xscrollcommand "$f.sbx set" \
            -insertofftime 400 -borderwidth 2
    set TkGnats(mtextbackround) [$f.text cget -background]
    $f.text configure -background grey95
    
    bind $f.text <Enter> "+focus $f.text"
    bind $f.text <Tab>   ""
    scrollbar $f.sby -command "$f.text yview" -relief sunken

    # Create padding based on the y scrollbar width and border
    frame $f.bottom
    scrollbar $f.sbx -command "$f.text xview" -borderwidth 2 -orient horizontal
    set pad [expr [$f.sby cget -width] + 2 * \
            ([$f.sby cget -bd] + [$f.sby cget -highlightthickness])]
    frame $f.pad -width $pad -height $pad

    pack  $f.pad    -in $f.bottom -side left        
    pack  $f.sbx    -in $f.bottom -side bottom -fill x
    pack  $f.bottom -side bottom -fill x 

    pack  $f.sby    -side left  -fill y
    pack  $f.text   -side right -fill both -expand true
    pack  $f        -side top   -fill both -expand true -pady 4
    
    return "$f.text"
}

proc flush_multitext {} {
    global current_multi_text frm
    set f .multiline
    if {"$current_multi_text" != ""} {
	set frm($current_multi_text) "[string trimright [$f.text get 1.0 end] "\t\n "]\n"
    }
}

proc flush_singletext {lst} {
    global frm
    foreach tag $lst {
	set frm($tag) [string trim [textget $tag] "\t\n "]
    }
}

proc switch_txt {name multitextflds} {
    global current_multi_text frm
    set f .multiline

    # write the current text out back into the frm bag
    flush_multitext

    # reset the currently selected button relief and command
    set num [lsearch $multitextflds $current_multi_text]
    if {$num >= 0} {
        .mb.b$num configure -relief raised \
                -command "switch_txt $current_multi_text {$multitextflds}"
    }

    # load the text widget with the new text
    if {[set state [$f.text cget -state]] == "disabled"} {
        $f.text configure -state normal
    }
    $f.text delete 1.0 end
    $f.text insert 1.0 $frm($name)
    if {$state == "disabled"} {
        $f.text configure -state disabled
    }

    # set the newly selected button relief and command
    set current_multi_text $name
    set num [lsearch $multitextflds $current_multi_text]
    .mb.b$num configure -relief sunken -command "helpMsg [string trimleft $current_multi_text >]"
}

proc save_sort_fields {sortfile sortfields} {
    if {$sortfile != ""} {
        check_tkgnats_userdir
        exec rm -f $sortfile
        set fout [open $sortfile "w"]
        puts  $fout "set Query(user_sort_flds) \{$sortfields\}"
        close $fout
    }
}

proc insert_file {w} {
    global TkGnats
    if {[info procs Fsel_Select] == ""} {
        source $TkGnats(lib)/fsel.tcl
    }
    set file [Fsel_Select "Enter filename to insert"]
    if {$file != ""} {
        set fin [open $file r]
        $w insert insert [read $fin]
        close $fin
    }
}

proc email_send {prid textw} {
    global TkGnats mail_cc mail_sj respval origval replval
    set addrs ""
    foreach a "$respval $origval $replval $mail_cc" {
        if {"[string trim $a]" != ""} {
            if {"$addrs" != ""} {
                set addrs "$addrs,"
            }
            set addrs "$addrs[string trim $a]"
        }
    }
    
    set   stamp  $TkGnats(LogName)[exec date +%j:%T]
    set   tmpfilename /tmp/tksendpr.$stamp
    set   fout [open $tmpfilename "w"]
    puts  $fout "To: $TkGnats(GNATS_ADDR)"
    if {"$addrs" != ""} {
        puts $fout "Cc: $addrs"
    }
    puts  $fout "From: $TkGnats(EmailAddr)"
    puts  $fout "Reply-To: $TkGnats(EmailAddr)"
    puts  $fout "Subject: Re: $prid: $mail_sj"
    puts  $fout "\n"
    puts  $fout "[$textw get 1.0 end]"
    close $fout

#####    exec $TkGnats(Mailer) -s "Re: $prid: $mail_sj" $addrs << "[$textw get 1.0 end]\n"
#    eval exec $TkGnats(Mailer) $addrs < $tmpfilename
    # sendmail errors are mailed back to sender
    set code [catch {eval exec $TkGnats(Mailer) < $tmpfilename} result]
    exec rm $tmpfilename
}

proc email_originator {to_rep to_res to_org prid synopsis} {
    global TkGnats mail_cc mail_sj respval origval replval env
##    set txt ""
##    append txt "To: $to_org\n"
##    append txt "Cc: $TkGnats(GNATS_ADDR)\n"
##    append txt "From: $TkGnats(EmailAddr)\n"
##    append txt "Subject: Re: $prid: $synopsis\n"
##    append txt "\n"

    # expand the Responsible address, which could be an alias in the responsibles file
    
    set responsible_code [catch {my_pr_addr $to_res} to_res_addr]
    if {$responsible_code} {
	set err "Could not verify E-mail address\nof responsible party.\n"
	append err "Address used: $to_res\n"
	append err "Resulting error(s):\n\n$to_res_addr\n"
	set to_res_addr ""
	Msg $err
	return -1
    }
    if {"$to_res" != "$to_res_addr"} {
        set restmp "$to_res: $to_res_addr"
    } {
        set restmp "$to_res"
    }
    set to_res "$to_res_addr"
 
    set tlist {}
    
    set t [toplevel .tkgnats_email]
    wm minsize    $t 100 100
    wm title      $t "TkGnats - Send Email to Problem Report: $prid"
    wm iconbitmap $t @$TkGnats(lib)/tkeditpr.xbm
    wm iconname   $t "$env(LOGNAME)'s tkemailpr [file tail $prid]"
    
    set    f [frame $t.f]
    pack  $f           -side top  -fill both -expand true

    frame $f.ad
    pack  $f.ad        -side top  -anchor n -fill x
    frame $f.ad.lab
    pack  $f.ad.lab    -side left -anchor w -fill y
    frame $f.ad.ent
    pack  $f.ad.ent    -side left -anchor w -fill x -expand true
    
    label $f.ad.lab.to   -relief flat -anchor e -text "To:" -width 8
    pack  $f.ad.lab.to   -side top    -anchor n

    label $f.ad.lab.sj   -relief flat -anchor e -text "Subject:"
    pack  $f.ad.lab.sj   -side bottom -anchor s -pady 2
    
    label $f.ad.lab.cc   -relief flat -anchor e -text "Cc:" -width 8
    pack  $f.ad.lab.cc   -side bottom -anchor s -pady 2
    
    label $f.ad.ent.to   -relief flat -anchor w -text "$TkGnats(GNATS_ADDR)"
    pack  $f.ad.ent.to   -side top    -anchor w
    
##    label $f.ad.ent.resp -relief flat -anchor w -text "Responsible ($to_res)"
##    pack  $f.ad.ent.resp -side top    -anchor w
    
    checkbutton $f.ad.ent.resp -relief flat -pady 0 -highlightthickness 0 -anchor w \
            -variable respval  -text "Responsible   ($restmp)" -offvalue "" -onvalue "$to_res"
    pack        $f.ad.ent.resp -side top -anchor w
    checkbutton $f.ad.ent.orig -relief flat -pady 0 -highlightthickness 0 -anchor w \
            -variable origval  -text "Originator    ($to_org)" -offvalue "" -onvalue "$to_org"
    pack        $f.ad.ent.orig -side top -anchor w
    checkbutton $f.ad.ent.repl -relief flat -pady 0 -highlightthickness 0 -anchor w \
            -variable replval  -text "Actual Sender ($to_rep)" -offvalue "" -onvalue "$to_rep"
    pack        $f.ad.ent.repl -side top -anchor w

    lappend tlist [entry $f.ad.ent.cc -relief sunken -borderwidth 2 -background grey95 \
            -textvariable mail_cc]
    pack  $f.ad.ent.cc -side top -anchor w -fill x -expand true
    bind  $f.ad.ent.cc <Enter> "+focus $f.ad.ent.cc"
    
    frame $f.ad.ent.sj
    pack  $f.ad.ent.sj -anchor w -fill x -expand true
    label $f.ad.ent.sj.def -relief sunken -anchor w -text "Re: $prid:"
    pack  $f.ad.ent.sj.def -side left
    lappend tlist [entry $f.ad.ent.sj.ent -relief sunken -borderwidth 2 -background grey95 \
            -textvariable mail_sj]
    pack  $f.ad.ent.sj.ent -side left -anchor w -fill x -expand true
    bind  $f.ad.ent.sj.ent <Enter> "+focus $f.ad.ent.sj.ent"

    scrollbar $f.sb -command "$f.text yview" -relief sunken
    lappend tlist [text $f.text \
	-wrap $TkGnats(TextWrap) \
	-yscrollcommand "$f.sb set" \
	-height 30 -width 80 -relief sunken -padx 4 -insertwidth 1 \
	-insertofftime 400 -borderwidth 2 -background grey95]
    bind  $f.text <Enter> "+focus $f.text"

    set_text_traversal $tlist
    
    pack $f.sb   -side left  -fill y
    pack $f.text -side right -expand true -fill both

    set     b [frame $t.b]
    pack   $b -side bottom -anchor s
    button $b.cancel -text Cancel -command "destroy $t"
    button $b.send   -text Send   -command "email_send $prid $f.text ; destroy $t"
    pack   $b.send   -side left -padx 60 -pady 10
    pack   $b.cancel -side left -padx 60 -pady 10

    focus $f.text

    if {$to_res != $TkGnats(EmailAddr)} {
        set respval $to_res
    } {
        set respval ""
        $f.ad.ent.resp configure -state disabled
    }
    if {$to_org != $TkGnats(EmailAddr) && $to_org != $to_res} {
        set origval $to_org
    } {
        set origval ""
        $f.ad.ent.orig configure -state disabled
    }
    if {$to_rep != $TkGnats(EmailAddr) && $to_rep != $to_res && $to_rep != $to_org} {
        set replval $to_rep
    } {
        set replval ""
        $f.ad.ent.repl configure -state disabled
    }
    set mail_cc ""
    set mail_sj "$synopsis"
}

proc entryDialog {msg {no_cancel ""} {initial_value ""}} {
    global  entryDialog_Value
    set entryDialog_Value ""
    catch  {destroy  .entryDialog}
    set t  [toplevel .entryDialog -borderwidth 2 -relief raised]
    message $t.msg -text $msg -aspect 99999
    entry   $t.e   -width 50 -borderwidth 2 -relief sunken
    frame   $t.bar 
    button  $t.bar.ok     -text "OK"     -command "set entryDialog_Value \[$t.e get\]"
    pack    $t.bar.ok -side left -padx 8 -pady 8
    if {$no_cancel == ""} {
        button  $t.bar.cancel -text "Cancel" -command "set entryDialog_Value {}"
        pack    $t.bar.cancel -side left -padx 8 -pady 8
    }
    pack    $t.msg -side top -fill x
    pack    $t.e   -side top -padx 8 -pady 8
    pack    $t.bar -side bottom -anchor center
    bind    $t.e <KeyPress-Return> "set entryDialog_Value \[$t.e get\]"
    bind    $t.e <Enter> "+focus $t.e"
    focus   $t.e
    $t.e insert 0 $initial_value
    set done 0
    while {$done == 0} {
        grab    $t
        tkwait  variable entryDialog_Value
        grab    release $t
        if {$no_cancel != "" && $entryDialog_Value == ""} {
            bell
            tk_dialog .entryDialog_complain "Blank entered." "You must enter a value." \
                    "warning" -1 "OK"
            set done 0
        } {
            set done 1
        }
    }
    destroy $t
    update idletasks
    return  $entryDialog_Value
}

proc quickfill_entry_from_listbox {ab ew lw vlist} {
    upvar 1 $ab tvar
    if {$tvar == ""} {
        return
    }
    set eidx   [$ew index insert]
    set tmpval [string range [$ew get] 0 [expr $eidx - 1]]
    if {$tmpval != ""} {
        set tvar $tmpval
    }
    set lidx [lsearch -regexp $vlist ^$tvar]
    if {$lidx < 0} {
        set tvar [string range [$ew get] 0 [expr $eidx - 2]]
        set lidx [lsearch -regexp $vlist ^$tvar]
        incr eidx -1
        bell
    }
    $lw selection clear 0 end
    if {$tvar != ""} {
        if {$lidx >= 0} {
            $lw selection set $lidx
            $lw see $lidx
            set tvar [$lw get [$lw curselection]]
        }
    }
    set tmpval   $tvar
    $ew delete 0 end
    $ew insert 0 $tmpval
    $ew icursor  $eidx
    if {$eidx > 0} {
        $ew selection range $eidx [string length $tmpval]
    }
}
